package com.sample;

import java.io.InputStreamReader;
import java.io.Reader;

import org.drools.RuleBase;
import org.drools.RuleBaseConfiguration;
import org.drools.RuleBaseFactory;
import org.drools.WorkingMemory;
import org.drools.compiler.PackageBuilder;
import org.drools.rule.Package;

import java.io.FileOutputStream;
import java.io.ObjectOutputStream;
import java.io.FileInputStream;
import java.util.Calendar;

/**
 * This is a sample file to launch a rule package from a rule source file.
 */
public class DroolsQueryTest {

	private static final int NO_OF_DATA_POINTS = 900;
	
    public static final void main(String[] args) {
        try {
        	
        	//load up the rulebase
        	
            RuleBase ruleBase = readRule();
            WorkingMemory workingMemory = null;

            workingMemory = ruleBase.newStatefulSession();
        	
            DataObject pdo = null;
            for (int i = 0; i < NO_OF_DATA_POINTS; i++) 
            {
            	pdo = 
                	new DataObject(10, new String("This is a Test Message."),Calendar.getInstance(),Calendar.getInstance());
            	
                
               workingMemory.insert( pdo );
				
			}
    
			////////////////////////////////////////////
			//
			//  This block serializes working memory
			//
			////////////////////////////////////////////
            try
            {

              long initial = Calendar.getInstance().getTimeInMillis();
              FileOutputStream wm_file = new
              		 FileOutputStream ("D:\\InfStorage\\WM"+NO_OF_DATA_POINTS+".data");
              ObjectOutputStream wm_out = new
              		 ObjectOutputStream (wm_file);
              System.out.println("Saveing WM....");
              wm_out.writeObject (workingMemory);
              
              System.out.println("Time required for Serializing WM : "+ (Calendar.getInstance().getTimeInMillis() - initial)/1000 + " sec. ");
              workingMemory = null;
              ruleBase = null;
              System.gc();
              System.gc();
            }
            catch (Exception e)
            {
              System.out.println (e.toString ());
              System.exit (1);
            }
           
			////////////////////////////////////////////
			//
			//              This block De serializes working memory
			//
			//////////////////////////////////////////            //   
            
            
            try
            {
              ruleBase = readRule();
              long initial = Calendar.getInstance().getTimeInMillis();
              FileInputStream wm_file = new
              		 FileInputStream ("D:\\InfStorage\\WM"+NO_OF_DATA_POINTS+".data");
              workingMemory = ruleBase.newStatefulSession(wm_file);
              System.out.println("Loaded WM...." + workingMemory);
                 
              System.out.println("Time required for De Serialize WM : "+ (Calendar.getInstance().getTimeInMillis() - initial)/1000 + " sec. ");
              
            }
            catch (Exception e)
            {
              System.out.println (e.toString ());
              System.exit (1);
            }
            
            
        } catch (Throwable t) {
            t.printStackTrace();
        }
    }

    /**
     * Please note that this is the "low level" rule assembly API.
     */
	private static RuleBase readRule() throws Exception {
		//read in the source
		Reader source = new InputStreamReader( DroolsQueryTest.class.getResourceAsStream( "QuerySample.drl" ) );
		
		//optionally read in the DSL (if you are using it).
		//Reader dsl = new InputStreamReader( DroolsTest.class.getResourceAsStream( "/mylang.dsl" ) );

		//Use package builder to build up a rule package.
		//An alternative lower level class called "DrlParser" can also be used...
		
		PackageBuilder builder = new PackageBuilder();

		//this wil parse and compile in one step
		//NOTE: There are 2 methods here, the one argument one is for normal DRL.
		builder.addPackageFromDrl( source );

		//Use the following instead of above if you are using a DSL:
		//builder.addPackageFromDrl( source, dsl );
		
		//get the compiled package (which is serializable)
		Package pkg = builder.getPackage();
		
		//add the package to a rulebase (deploy the rule package).
		RuleBaseConfiguration rbc = new RuleBaseConfiguration();
    	rbc.setShadowProxy(false);
		RuleBase ruleBase = RuleBaseFactory.newRuleBase(rbc);
		ruleBase.addPackage( pkg );
		return ruleBase;
	}

    
}
